@@ -1,4 +1,4 @@
-/*	$OpenBSD: config.c,v 1.90 2022/11/18 18:56:39 mbuhl Exp $	*/
+/*	$OpenBSD: config.c,v 1.93 2023/08/04 19:06:25 claudio Exp $	*/
 
 /*
  * Copyright (c) 2019-2021 Tobias Heider <tobhe@openbsd.org>
@@ -86,12 +86,12 @@ config_free_kex(struct iked_kex *kex)
 	if (kex == NULL)
 		return;
 
-	ibuf_release(kex->kex_inonce);
-	ibuf_release(kex->kex_rnonce);
+	ibuf_free(kex->kex_inonce);
+	ibuf_free(kex->kex_rnonce);
 
 	group_free(kex->kex_dhgroup);
-	ibuf_release(kex->kex_dhiexchange);
-	ibuf_release(kex->kex_dhrexchange);
+	ibuf_free(kex->kex_dhiexchange);
+	ibuf_free(kex->kex_dhrexchange);
 
 	free(kex);
 }
@@ -144,42 +144,42 @@ config_free_sa(struct iked *env, struct iked_sa *sa)
 	ikev2_msg_flushqueue(env, &sa->sa_requests);
 	ikev2_msg_flushqueue(env, &sa->sa_responses);
 
-	ibuf_release(sa->sa_inonce);
-	ibuf_release(sa->sa_rnonce);
+	ibuf_free(sa->sa_inonce);
+	ibuf_free(sa->sa_rnonce);
 
 	group_free(sa->sa_dhgroup);
-	ibuf_release(sa->sa_dhiexchange);
-	ibuf_release(sa->sa_dhrexchange);
+	ibuf_free(sa->sa_dhiexchange);
+	ibuf_free(sa->sa_dhrexchange);
 
-	ibuf_release(sa->sa_simult);
+	ibuf_free(sa->sa_simult);
 
 	hash_free(sa->sa_prf);
 	hash_free(sa->sa_integr);
 	cipher_free(sa->sa_encr);
 
-	ibuf_release(sa->sa_key_d);
-	ibuf_release(sa->sa_key_iauth);
-	ibuf_release(sa->sa_key_rauth);
-	ibuf_release(sa->sa_key_iencr);
-	ibuf_release(sa->sa_key_rencr);
-	ibuf_release(sa->sa_key_iprf);
-	ibuf_release(sa->sa_key_rprf);
+	ibuf_free(sa->sa_key_d);
+	ibuf_free(sa->sa_key_iauth);
+	ibuf_free(sa->sa_key_rauth);
+	ibuf_free(sa->sa_key_iencr);
+	ibuf_free(sa->sa_key_rencr);
+	ibuf_free(sa->sa_key_iprf);
+	ibuf_free(sa->sa_key_rprf);
 
-	ibuf_release(sa->sa_1stmsg);
-	ibuf_release(sa->sa_2ndmsg);
+	ibuf_free(sa->sa_1stmsg);
+	ibuf_free(sa->sa_2ndmsg);
 
-	ibuf_release(sa->sa_iid.id_buf);
-	ibuf_release(sa->sa_rid.id_buf);
-	ibuf_release(sa->sa_icert.id_buf);
-	ibuf_release(sa->sa_rcert.id_buf);
+	ibuf_free(sa->sa_iid.id_buf);
+	ibuf_free(sa->sa_rid.id_buf);
+	ibuf_free(sa->sa_icert.id_buf);
+	ibuf_free(sa->sa_rcert.id_buf);
 	for (i = 0; i < IKED_SCERT_MAX; i++)
-		ibuf_release(sa->sa_scert[i].id_buf);
-	ibuf_release(sa->sa_localauth.id_buf);
-	ibuf_release(sa->sa_peerauth.id_buf);
+		ibuf_free(sa->sa_scert[i].id_buf);
+	ibuf_free(sa->sa_localauth.id_buf);
+	ibuf_free(sa->sa_peerauth.id_buf);
 
-	ibuf_release(sa->sa_eap.id_buf);
+	ibuf_free(sa->sa_eap.id_buf);
 	free(sa->sa_eapid);
-	ibuf_release(sa->sa_eapmsk);
+	ibuf_free(sa->sa_eapmsk);
 
 	free(sa->sa_cp_addr);
 	free(sa->sa_cp_addr6);
@@ -973,11 +973,11 @@ config_setocsp(struct iked *env)
 int
 config_getocsp(struct iked *env, struct imsg *imsg)
 {
-	size_t			 have, need;
-	u_int8_t		*ptr;
+	size_t		 have, need;
+	uint8_t		*ptr;
 
 	free(env->sc_ocsp_url);
-	ptr = (u_int8_t *)imsg->data;
+	ptr = (uint8_t *)imsg->data;
 	have = IMSG_DATA_SIZE(imsg);
 
 	/* get tolerate */
@@ -1065,7 +1065,7 @@ config_setkeys(struct iked *env)
 	iov[0].iov_base = &privkey;
 	iov[0].iov_len = sizeof(privkey);
 	iov[1].iov_base = ibuf_data(privkey.id_buf);
-	iov[1].iov_len = ibuf_length(privkey.id_buf);
+	iov[1].iov_len = ibuf_size(privkey.id_buf);
 
 	if (proc_composev(&env->sc_ps, PROC_CERT, IMSG_PRIVKEY, iov, 2) == -1) {
 		log_warnx("%s: failed to send private key", __func__);
@@ -1075,7 +1075,7 @@ config_setkeys(struct iked *env)
 	iov[0].iov_base = &pubkey;
 	iov[0].iov_len = sizeof(pubkey);
 	iov[1].iov_base = ibuf_data(pubkey.id_buf);
-	iov[1].iov_len = ibuf_length(pubkey.id_buf);
+	iov[1].iov_len = ibuf_size(pubkey.id_buf);
 
 	if (proc_composev(&env->sc_ps, PROC_CERT, IMSG_PUBKEY, iov, 2) == -1) {
 		log_warnx("%s: failed to send public key", __func__);
@@ -1087,8 +1087,8 @@ config_setkeys(struct iked *env)
 	if (fp != NULL)
 		fclose(fp);
 
-	ibuf_release(pubkey.id_buf);
-	ibuf_release(privkey.id_buf);
+	ibuf_free(pubkey.id_buf);
+	ibuf_free(privkey.id_buf);
 	EVP_PKEY_free(key);
 
 	return (ret);
