@@ -1,4 +1,4 @@
-/*	$OpenBSD: dh.c,v 1.31 2021/12/13 18:06:56 tb Exp $	*/
+/*	$OpenBSD: dh.c,v 1.33 2023/07/28 07:31:38 claudio Exp $	*/
 
 /*
  * Copyright (c) 2020-2021 Tobias Heider <tobhe@openbsd.org>
@@ -402,7 +402,7 @@ dh_create_exchange(struct dh_group *group, struct ibuf **bufp, struct ibuf *iexc
 	if (buf == NULL)
 		return -1;
 	*bufp = buf;
-	return (group->exchange(group, buf->buf));
+	return (group->exchange(group, ibuf_data(buf)));
 }
 
 int
@@ -420,7 +420,7 @@ dh_create_shared(struct dh_group *group, struct ibuf **secretp, struct ibuf *exc
 	if (buf == NULL)
 		return -1;
 	*secretp = buf;
-	return (group->shared(group, buf->buf, exchange->buf));
+	return (group->shared(group, ibuf_data(buf), ibuf_data(exchange)));
 }
 
 int
@@ -802,7 +802,7 @@ kemsx_create_exchange2(struct dh_group *group, struct ibuf **bufp,
 		buf = ibuf_new(NULL, need);
 		if (buf == NULL)
 			return -1;
-		cp = buf->buf;
+		cp = ibuf_data(buf);
 		memcpy(cp, kemsx->public,
 		    crypto_kem_sntrup761_PUBLICKEYBYTES);
 		cp += crypto_kem_sntrup761_PUBLICKEYBYTES;
@@ -820,8 +820,8 @@ kemsx_create_exchange2(struct dh_group *group, struct ibuf **bufp,
 		buf = ibuf_new(NULL, need);
 		if (buf == NULL)
 			return -1;
-		cp = buf->buf;
-		pk = iexchange->buf;
+		cp = ibuf_data(buf);
+		pk = ibuf_data(iexchange);
 		crypto_kem_sntrup761_enc(cp, kemsx->kemkey, pk);
 		cp += crypto_kem_sntrup761_CIPHERTEXTBYTES;
 	}
@@ -839,8 +839,8 @@ kemsx_create_shared2(struct dh_group *group, struct ibuf **sharedp,
 	struct kemsx_key	*kemsx = group->kemsx;
 	struct ibuf		*buf = NULL;
 	EVP_MD_CTX		*ctx = NULL;
-	u_int8_t		*cp;
-	u_int8_t		 shared[CURVE25519_SIZE];
+	uint8_t			*cp;
+	uint8_t			 shared[CURVE25519_SIZE];
 	size_t			 have, need;
 	u_int			 len;
 
@@ -851,7 +851,7 @@ kemsx_create_shared2(struct dh_group *group, struct ibuf **sharedp,
 		return (-1);
 
 	have = ibuf_size(exchange);
-	cp = exchange->buf;
+	cp = ibuf_data(exchange);
 	if (kemsx->initiator) {
 		/* input */
 		need = crypto_kem_sntrup761_CIPHERTEXTBYTES +
@@ -879,7 +879,7 @@ kemsx_create_shared2(struct dh_group *group, struct ibuf **sharedp,
 	    EVP_DigestInit_ex(ctx, EVP_sha512(), NULL) != 1 ||
 	    EVP_DigestUpdate(ctx, kemsx->kemkey, sizeof(kemsx->kemkey)) != 1 ||
 	    EVP_DigestUpdate(ctx, shared, sizeof(shared)) != 1 ||
-	    EVP_DigestFinal_ex(ctx, buf->buf, &len) != 1) {
+	    EVP_DigestFinal_ex(ctx, ibuf_data(buf), &len) != 1) {
 		EVP_MD_CTX_free(ctx);
 		ibuf_free(buf);
 		return (-1);
