@@ -1,4 +1,4 @@
-/*	$OpenBSD: eap.c,v 1.21 2022/07/08 19:51:11 tobhe Exp $	*/
+/*	$OpenBSD: eap.c,v 1.25 2023/07/18 15:07:41 claudio Exp $	*/
 
 /*
  * Copyright (c) 2010-2013 Reyk Floeter <reyk@openbsd.org>
@@ -28,6 +28,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <signal.h>
+#include <endian.h>
 #include <errno.h>
 #include <err.h>
 #include <event.h>
@@ -50,7 +51,7 @@ eap_add_id_request(struct ibuf *e)
 {
 	struct eap_message		*eap;
 
-	if ((eap = ibuf_advance(e, sizeof(*eap))) == NULL)
+	if ((eap = ibuf_reserve(e, sizeof(*eap))) == NULL)
 		return (-1);
 	eap->eap_code = EAP_CODE_REQUEST;
 	eap->eap_id = 0;
@@ -111,7 +112,7 @@ eap_identity_request(struct iked *env, struct iked_sa *sa)
 	if ((pld = ikev2_add_payload(e)) == NULL)
 		goto done;
 	firstpayload = IKEV2_PAYLOAD_IDr;
-	if (ibuf_cat(e, id->id_buf) != 0)
+	if (ibuf_add_buf(e, id->id_buf) != 0)
 		goto done;
 	len = ibuf_size(id->id_buf);
 
@@ -123,10 +124,10 @@ eap_identity_request(struct iked *env, struct iked_sa *sa)
 		/* CERT payload */
 		if ((pld = ikev2_add_payload(e)) == NULL)
 			goto done;
-		if ((cert = ibuf_advance(e, sizeof(*cert))) == NULL)
+		if ((cert = ibuf_reserve(e, sizeof(*cert))) == NULL)
 			goto done;
 		cert->cert_type = certid->id_type;
-		if (ibuf_cat(e, certid->id_buf) != 0)
+		if (ibuf_add_buf(e, certid->id_buf) != 0)
 			goto done;
 		len = ibuf_size(certid->id_buf) + sizeof(*cert);
 
@@ -138,10 +139,10 @@ eap_identity_request(struct iked *env, struct iked_sa *sa)
 				goto done;
 			if ((pld = ikev2_add_payload(e)) == NULL)
 				goto done;
-			if ((cert = ibuf_advance(e, sizeof(*cert))) == NULL)
+			if ((cert = ibuf_reserve(e, sizeof(*cert))) == NULL)
 				goto done;
 			cert->cert_type = sa->sa_scert[i].id_type;
-			if (ibuf_cat(e, sa->sa_scert[i].id_buf) != 0)
+			if (ibuf_add_buf(e, sa->sa_scert[i].id_buf) != 0)
 				goto done;
 			len = ibuf_size(sa->sa_scert[i].id_buf) + sizeof(*cert);
 		}
@@ -153,10 +154,10 @@ eap_identity_request(struct iked *env, struct iked_sa *sa)
 	/* AUTH payload */
 	if ((pld = ikev2_add_payload(e)) == NULL)
 		goto done;
-	if ((auth = ibuf_advance(e, sizeof(*auth))) == NULL)
+	if ((auth = ibuf_reserve(e, sizeof(*auth))) == NULL)
 		goto done;
 	auth->auth_method = sa->sa_localauth.id_type;
-	if (ibuf_cat(e, sa->sa_localauth.id_buf) != 0)
+	if (ibuf_add_buf(e, sa->sa_localauth.id_buf) != 0)
 		goto done;
 	len = ibuf_size(sa->sa_localauth.id_buf) + sizeof(*auth);
 
@@ -175,7 +176,7 @@ eap_identity_request(struct iked *env, struct iked_sa *sa)
 	ret = ikev2_msg_send_encrypt(env, sa, &e,
 	    IKEV2_EXCHANGE_IKE_AUTH, firstpayload, 1);
  done:
-	ibuf_release(e);
+	ibuf_free(e);
 	return (ret);
 }
 
@@ -192,7 +193,7 @@ eap_challenge_request(struct iked *env, struct iked_sa *sa,
 	if ((e = ibuf_static()) == NULL)
 		return (-1);
 
-	if ((eap = ibuf_advance(e, sizeof(*eap))) == NULL)
+	if ((eap = ibuf_reserve(e, sizeof(*eap))) == NULL)
 		goto done;
 	eap->eap_code = EAP_CODE_REQUEST;
 	eap->eap_id = eap_id + 1;
@@ -204,7 +205,7 @@ eap_challenge_request(struct iked *env, struct iked_sa *sa,
 		eap->eap_length = htobe16(sizeof(*eap) +
 		    sizeof(*ms) + strlen(name));
 
-		if ((ms = ibuf_advance(e, sizeof(*ms))) == NULL)
+		if ((ms = ibuf_reserve(e, sizeof(*ms))) == NULL)
 			return (-1);
 		ms->msc_opcode = EAP_MSOPCODE_CHALLENGE;
 		ms->msc_id = eap->eap_id;
@@ -229,7 +230,7 @@ eap_challenge_request(struct iked *env, struct iked_sa *sa,
 	ret = ikev2_send_ike_e(env, sa, e,
 	    IKEV2_PAYLOAD_EAP, IKEV2_EXCHANGE_IKE_AUTH, 1);
  done:
-	ibuf_release(e);
+	ibuf_free(e);
 	return (ret);
 }
 
@@ -243,7 +244,7 @@ eap_message_send(struct iked *env, struct iked_sa *sa, int eap_code, int eap_id)
 	if ((e = ibuf_static()) == NULL)
 		return (-1);
 
-	if ((resp = ibuf_advance(e, sizeof(*resp))) == NULL)
+	if ((resp = ibuf_reserve(e, sizeof(*resp))) == NULL)
 		goto done;
 	resp->eap_code = eap_code;
 	resp->eap_id = eap_id;
@@ -252,7 +253,7 @@ eap_message_send(struct iked *env, struct iked_sa *sa, int eap_code, int eap_id)
 	ret = ikev2_send_ike_e(env, sa, e,
 	    IKEV2_PAYLOAD_EAP, IKEV2_EXCHANGE_IKE_AUTH, 1);
  done:
-	ibuf_release(e);
+	ibuf_free(e);
 	return (ret);
 }
 
@@ -277,7 +278,7 @@ eap_mschap_challenge(struct iked *env, struct iked_sa *sa, int eap_id,
 
 	msg = " M=Welcome";
 
-	if ((resp = ibuf_advance(eapmsg, sizeof(*resp))) == NULL)
+	if ((resp = ibuf_reserve(eapmsg, sizeof(*resp))) == NULL)
 		goto done;
 	resp->eap_code = EAP_CODE_REQUEST;
 	resp->eap_id = eap_id + 1;
@@ -285,7 +286,7 @@ eap_mschap_challenge(struct iked *env, struct iked_sa *sa, int eap_id,
 	    success_size + strlen(msg));
 	resp->eap_type = EAP_TYPE_MSCHAP_V2;
 
-	if ((mss = ibuf_advance(eapmsg, sizeof(*mss))) == NULL)
+	if ((mss = ibuf_reserve(eapmsg, sizeof(*mss))) == NULL)
 		goto done;
 	mss->mss_opcode = EAP_MSOPCODE_SUCCESS;
 	mss->mss_id = msr_id;
@@ -299,7 +300,7 @@ eap_mschap_challenge(struct iked *env, struct iked_sa *sa, int eap_id,
 	ret = ikev2_send_ike_e(env, sa, eapmsg,
 	    IKEV2_PAYLOAD_EAP, IKEV2_EXCHANGE_IKE_AUTH, 1);
  done:
-	ibuf_release(eapmsg);
+	ibuf_free(eapmsg);
 	return (ret);
 }
 
@@ -313,20 +314,20 @@ eap_mschap_success(struct iked *env, struct iked_sa *sa, int eap_id)
 
 	if ((eapmsg = ibuf_static()) == NULL)
 		return (-1);
-	if ((resp = ibuf_advance(eapmsg, sizeof(*resp))) == NULL)
+	if ((resp = ibuf_reserve(eapmsg, sizeof(*resp))) == NULL)
 		goto done;
 	resp->eap_code = EAP_CODE_RESPONSE;
 	resp->eap_id = eap_id;
 	resp->eap_length = htobe16(sizeof(*resp) + sizeof(*ms));
 	resp->eap_type = EAP_TYPE_MSCHAP_V2;
-	if ((ms = ibuf_advance(eapmsg, sizeof(*ms))) == NULL)
+	if ((ms = ibuf_reserve(eapmsg, sizeof(*ms))) == NULL)
 		goto done;
 	ms->ms_opcode = EAP_MSOPCODE_SUCCESS;
 
 	ret = ikev2_send_ike_e(env, sa, eapmsg,
 	    IKEV2_PAYLOAD_EAP, IKEV2_EXCHANGE_IKE_AUTH, 1);
  done:
-	ibuf_release(eapmsg);
+	ibuf_free(eapmsg);
 	return (ret);
 }
 
