@@ -1,4 +1,4 @@
-/*	$OpenBSD: ikev2_pld.c,v 1.125 2022/09/19 20:54:02 tobhe Exp $	*/
+/*	$OpenBSD: ikev2_pld.c,v 1.133 2023/09/02 18:36:30 tobhe Exp $	*/
 
 /*
  * Copyright (c) 2019 Tobias Heider <tobias.heider@stusta.de>
@@ -30,6 +30,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <signal.h>
+#include <endian.h>
 #include <errno.h>
 #include <err.h>
 #include <event.h>
@@ -319,7 +320,7 @@ ikev2_validate_sa(struct iked_message *msg, size_t offset, size_t left,
 	 */
 	if (!sap->sap_more && left != sap_length) {
 		log_debug("%s: malformed payload: SA payload length mismatches "
-		    "single proposal substructure length (%lu != %zu)",
+		    "single proposal substructure length (%zu != %zu)",
 		    __func__, left, sap_length);
 		return (-1);
 	}
@@ -684,7 +685,7 @@ ikev2_pld_ke(struct iked *env, struct ikev2_payload *pld,
 	print_hex(buf, 0, len);
 
 	if (ikev2_msg_frompeer(msg)) {
-		if (ibuf_length(msg->msg_parent->msg_ke)) {
+		if (msg->msg_parent->msg_ke != NULL) {
 			log_info("%s: duplicate KE payload", __func__);
 			return (-1);
 		}
@@ -747,7 +748,7 @@ ikev2_pld_id(struct iked *env, struct ikev2_payload *pld,
 		return (-1);
 
 	if (ikev2_print_id(&idb, idstr, sizeof(idstr)) == -1) {
-		ibuf_release(idb.id_buf);
+		ibuf_free(idb.id_buf);
 		log_debug("%s: malformed id", __func__);
 		return (-1);
 	}
@@ -755,7 +756,7 @@ ikev2_pld_id(struct iked *env, struct ikev2_payload *pld,
 	log_debug("%s: id %s length %zu", __func__, idstr, len);
 
 	if (!ikev2_msg_frompeer(msg)) {
-		ibuf_release(idb.id_buf);
+		ibuf_free(idb.id_buf);
 		return (0);
 	}
 
@@ -765,13 +766,13 @@ ikev2_pld_id(struct iked *env, struct ikev2_payload *pld,
 	else if (!sa->sa_hdr.sh_initiator && payload == IKEV2_PAYLOAD_IDr)
 		idp = &msg->msg_parent->msg_localid;
 	else {
-		ibuf_release(idb.id_buf);
+		ibuf_free(idb.id_buf);
 		log_debug("%s: unexpected id payload", __func__);
 		return (0);
 	}
 
 	if (idp->id_type) {
-		ibuf_release(idb.id_buf);
+		ibuf_free(idb.id_buf);
 		log_debug("%s: duplicate id payload", __func__);
 		return (-1);
 	}
@@ -795,6 +796,10 @@ ikev2_validate_cert(struct iked_message *msg, size_t offset, size_t left,
 		return (-1);
 	}
 	memcpy(cert, msgbuf + offset, sizeof(*cert));
+	if (cert->cert_type == IKEV2_CERT_NONE) {
+		log_debug("%s: malformed payload: invalid cert type", __func__);
+		return (-1);
+	}
 
 	return (0);
 }
@@ -809,6 +814,7 @@ ikev2_pld_cert(struct iked *env, struct ikev2_payload *pld,
 	struct iked_id			*certid;
 	uint8_t				*msgbuf = ibuf_data(msg->msg_data);
 	const struct iked_sa		*sa = msg->msg_sa;
+	int				 i;
 
 	if (ikev2_validate_cert(msg, offset, left, &cert))
 		return (-1);
@@ -825,13 +831,28 @@ ikev2_pld_cert(struct iked *env, struct ikev2_payload *pld,
 	if (!ikev2_msg_frompeer(msg))
 		return (0);
 
-	certid = &msg->msg_parent->msg_cert;
-	if (certid->id_type) {
-		log_debug("%s: multiple cert payloads, ignoring",
+	/* do not accept internal encoding in the wire */
+	if (cert.cert_type == IKEV2_CERT_BUNDLE) {
+		log_debug("%s: ignoring IKEV2_CERT_BUNDLE",
 		   SPI_SA(sa, __func__));
 		return (0);
 	}
 
+	certid = &msg->msg_parent->msg_cert;
+	if (certid->id_type) {
+		/* try to set supplemental certs */
+		for (i = 0; i < IKED_SCERT_MAX; i++) {
+			certid = &msg->msg_parent->msg_scert[i];
+			if (!certid->id_type)
+				break;
+		}
+		if (certid->id_type) {
+			log_debug("%s: too many cert payloads, ignoring",
+			   SPI_SA(sa, __func__));
+			return (0);
+		}
+	}
+
 	if ((certid->id_buf = ibuf_new(buf, len)) == NULL) {
 		log_debug("%s: failed to save cert", __func__);
 		return (-1);
@@ -963,7 +984,7 @@ ikev2_pld_auth(struct iked *env, struct ikev2_payload *pld,
 		return (-1);
 	}
 
-	ibuf_release(idp->id_buf);
+	ibuf_free(idp->id_buf);
 	idp->id_type = auth.auth_method;
 	idp->id_offset = 0;
 	if ((idp->id_buf = ibuf_new(buf, len)) == NULL)
@@ -991,7 +1012,7 @@ ikev2_pld_nonce(struct iked *env, struct ikev2_payload *pld,
 	print_hex(buf, 0, len);
 
 	if (ikev2_msg_frompeer(msg)) {
-		if (ibuf_length(msg->msg_parent->msg_nonce)) {
+		if (msg->msg_parent->msg_nonce != NULL) {
 			log_info("%s: duplicate NONCE payload", __func__);
 			return (-1);
 		}
@@ -1278,7 +1299,7 @@ ikev2_pld_notify(struct iked *env, struct ikev2_payload *pld,
 			    " notification: %zu", __func__, left);
 			return (0);
 		}
-		ibuf_release(msg->msg_cookie2);	/* should not happen */
+		ibuf_free(msg->msg_cookie2);	/* should not happen */
 		if ((msg->msg_cookie2 = ibuf_new(buf, left)) == NULL) {
 			log_debug("%s: failed to get peer cookie2", __func__);
 			return (-1);
@@ -1299,7 +1320,7 @@ ikev2_pld_notify(struct iked *env, struct ikev2_payload *pld,
 		log_debug("%s: received cookie, len %zu", __func__, left);
 		print_hex(buf, 0, left);
 
-		ibuf_release(msg->msg_cookie);
+		ibuf_free(msg->msg_cookie);
 		if ((msg->msg_cookie = ibuf_new(buf, left)) == NULL) {
 			log_debug("%s: failed to get peer cookie", __func__);
 			return (-1);
@@ -1521,12 +1542,12 @@ int
 ikev2_pld_ts(struct iked *env, struct ikev2_payload *pld,
     struct iked_message *msg, size_t offset, size_t left, unsigned int type)
 {
-	struct sockaddr_in		 s4;
-	struct sockaddr_in6		 s6;
-	uint8_t				 buf[2][128];
+	struct sockaddr_in		 start4, end4;
+	struct sockaddr_in6		 start6, end6;
+	uint8_t				*msgbuf = ibuf_data(msg->msg_data);
 	uint8_t				*ptr;
 
-	ptr = ibuf_data(msg->msg_data) + offset;
+	ptr = msgbuf + offset;
 
 	switch (type) {
 	case IKEV2_TS_IPV4_ADDR_RANGE:
@@ -1537,24 +1558,25 @@ ikev2_pld_ts(struct iked *env, struct ikev2_payload *pld,
 			return (-1);
 		}
 
-		bzero(&s4, sizeof(s4));
-		s4.sin_family = AF_INET;
+		bzero(&start4, sizeof(start4));
+		start4.sin_family = AF_INET;
 #ifdef HAVE_SOCKADDR_SA_LEN
-		s4.sin_len = sizeof(s4);
+		start4.sin_len = sizeof(start4);
 #endif
-		memcpy(&s4.sin_addr.s_addr, ptr, 4);
+		memcpy(&start4.sin_addr.s_addr, ptr, 4);
 		ptr += 4;
 		left -= 4;
-		print_host((struct sockaddr *)&s4,
-		    (char *)buf[0], sizeof(buf[0]));
 
-		memcpy(&s4.sin_addr.s_addr, ptr, 4);
+		bzero(&end4, sizeof(end4));
+		end4.sin_family = AF_INET;
+#ifdef HAVE_SOCKADDR_SA_LEN
+		end4.sin_len = sizeof(end4);
+#endif
+		memcpy(&end4.sin_addr.s_addr, ptr, 4);
 		left -= 4;
-		print_host((struct sockaddr *)&s4,
-		    (char *)buf[1], sizeof(buf[1]));
 
 		log_debug("%s: start %s end %s", __func__,
-		    buf[0], buf[1]);
+		    print_addr(&start4), print_addr(&end4));
 		break;
 	case IKEV2_TS_IPV6_ADDR_RANGE:
 		if (left < 2 * 16) {
@@ -1563,23 +1585,25 @@ ikev2_pld_ts(struct iked *env, struct ikev2_payload *pld,
 			    __func__, left, 2 * 16);
 			return (-1);
 		}
-		bzero(&s6, sizeof(s6));
-		s6.sin6_family = AF_INET6;
+		bzero(&start6, sizeof(start6));
+		start6.sin6_family = AF_INET6;
 #ifdef HAVE_SOCKADDR_SA_LEN
-		s6.sin6_len = sizeof(s6);
+		start6.sin6_len = sizeof(start6);
 #endif
-		memcpy(&s6.sin6_addr, ptr, 16);
+		memcpy(&start6.sin6_addr, ptr, 16);
 		ptr += 16;
 		left -= 16;
-		print_host((struct sockaddr *)&s6,
-		    (char *)buf[0], sizeof(buf[0]));
 
-		memcpy(&s6.sin6_addr, ptr, 16);
+		bzero(&end6, sizeof(end6));
+		end6.sin6_family = AF_INET6;
+#ifdef HAVE_SOCKADDR_SA_LEN
+		end6.sin6_len = sizeof(end6);
+#endif
+		memcpy(&end6.sin6_addr, ptr, 16);
 		left -= 16;
-		print_host((struct sockaddr *)&s6,
-		    (char *)buf[1], sizeof(buf[1]));
+
 		log_debug("%s: start %s end %s", __func__,
-		    buf[0], buf[1]);
+		    print_addr(&start6), print_addr(&end6));
 		break;
 	default:
 		log_debug("%s: ignoring unknown TS type %u", __func__, type);
@@ -1653,7 +1677,7 @@ ikev2_pld_ef(struct iked *env, struct ikev2_payload *pld,
 		    __func__, frag_num, frag_total);
 		goto done;
 	}
-	elen = ibuf_length(e);
+	elen = ibuf_size(e);
 
 	/* Check new fragmented message */
 	if (sa_frag->frag_arr == NULL) {
@@ -1713,12 +1737,12 @@ ikev2_pld_ef(struct iked *env, struct ikev2_payload *pld,
 done:
 	if (!processed)
 		ikestat_inc(env, ikes_frag_rcvd_drop);
-	ibuf_release(e);
+	ibuf_free(e);
 	return (ret);
 dropall:
 	ikestat_add(env, ikes_frag_rcvd_drop, sa_frag->frag_count + 1);
 	config_free_fragments(sa_frag);
-	ibuf_release(e);
+	ibuf_free(e);
 	return -1;
 }
 
@@ -1785,7 +1809,7 @@ done:
 	else
 		ikestat_add(env, ikes_frag_reass_drop, sa_frag->frag_total);
 	config_free_fragments(sa_frag);
-	ibuf_release(e);
+	ibuf_free(e);
 
 	return (ret);
 }
@@ -1840,7 +1864,7 @@ ikev2_pld_e(struct iked *env, struct ikev2_payload *pld,
 	    pld->pld_nextpayload);
 
  done:
-	ibuf_release(e);
+	ibuf_free(e);
 
 	return (ret);
 }
@@ -1870,15 +1894,15 @@ ikev2_pld_cp(struct iked *env, struct ikev2_payload *pld,
 	struct iked_addr	*addr;
 	struct sockaddr_in	*in4;
 	struct sockaddr_in6	*in6;
+	uint8_t			*msgbuf = ibuf_data(msg->msg_data);
 	uint8_t			*ptr;
 	size_t			 len;
-	uint8_t			 buf[128];
 	int			 cfg_type;
 
 	if (ikev2_validate_cp(msg, offset, left, &cp))
 		return (-1);
 
-	ptr = ibuf_data(msg->msg_data) + offset + sizeof(cp);
+	ptr = msgbuf + offset + sizeof(cp);
 	len = left - sizeof(cp);
 
 	log_debug("%s: type %s length %zu",
@@ -1952,17 +1976,20 @@ ikev2_pld_cp(struct iked *env, struct ikev2_payload *pld,
 			in4->sin_len = sizeof(*in4);
 #endif
 			memcpy(&in4->sin_addr.s_addr, ptr, 4);
-			print_host((struct sockaddr *)in4, (char *)buf,
-			    sizeof(buf));
-			log_debug("%s: cfg %s", __func__, buf);
 			switch(cfg_type) {
 			case IKEV2_CFG_INTERNAL_IP4_ADDRESS:
 				msg->msg_parent->msg_cp_addr = addr;
-				log_debug("%s: IP4_ADDRESS %s", __func__, buf);
+				log_debug("%s: IP4_ADDRESS %s", __func__,
+				    print_addr(&addr->addr));
 				break;
 			case IKEV2_CFG_INTERNAL_IP4_DNS:
 				msg->msg_parent->msg_cp_dns = addr;
-				log_debug("%s: IP4_DNS %s", __func__, buf);
+				log_debug("%s: IP4_DNS %s", __func__,
+				    print_addr(&addr->addr));
+				break;
+			default:
+				log_debug("%s: cfg %s", __func__,
+				    print_addr(&addr->addr));
 				break;
 			}
 			break;
@@ -2004,17 +2031,20 @@ ikev2_pld_cp(struct iked *env, struct ikev2_payload *pld,
 			in6->sin6_len = sizeof(*in6);
 #endif
 			memcpy(&in6->sin6_addr, ptr, 16);
-			print_host((struct sockaddr *)in6, (char *)buf,
-			    sizeof(buf));
-			log_debug("%s: cfg %s/%d", __func__, buf, ptr[16]);
 			switch(cfg_type) {
 			case IKEV2_CFG_INTERNAL_IP6_ADDRESS:
 				msg->msg_parent->msg_cp_addr6 = addr;
-				log_debug("%s: IP6_ADDRESS %s", __func__, buf);
+				log_debug("%s: IP6_ADDRESS %s", __func__,
+				    print_addr(&addr->addr));
 				break;
 			case IKEV2_CFG_INTERNAL_IP6_DNS:
 				msg->msg_parent->msg_cp_dns = addr;
-				log_debug("%s: IP6_DNS %s", __func__, buf);
+				log_debug("%s: IP6_DNS %s", __func__,
+				    print_addr(&addr->addr));
+				break;
+			default:
+				log_debug("%s: cfg %s/%d", __func__,
+				    print_addr(&addr->addr), ptr[16]);
 				break;
 			}
 			break;
