@@ -31,22 +31,32 @@
 #include <string.h>
 #include <unistd.h>
 
-#include <iked.h>
+#ifdef WITH_SYSTEMD
+#include "systemd/sd-bus.h"
+#endif
+
+#include "iked.h"
 
 int vroute_setroute(struct iked *, uint32_t, struct sockaddr *, uint8_t,
     struct sockaddr *, int);
 int vroute_doroute(struct iked *, uint8_t, struct sockaddr *,
     struct sockaddr *, struct sockaddr *, int);
 int vroute_doaddr(struct iked *, int, struct sockaddr *, struct sockaddr *, int);
-int vroute_dodns(struct iked *, struct sockaddr *, int, unsigned int);
+int vroute_dodns(struct iked *, int, unsigned int);
 void vroute_cleanup(struct iked *);
 
 void vroute_insertaddr(struct iked *, int, struct sockaddr *, struct sockaddr *);
 void vroute_removeaddr(struct iked *, int, struct sockaddr *, struct sockaddr *);
-void vroute_insertiface(struct iked *, unsigned int);
-void vroute_removeiface(struct iked *, unsigned int);
+void vroute_insertdns(struct iked *, int, struct sockaddr *);
+void vroute_removedns(struct iked *, int, struct sockaddr *);
 void vroute_insertroute(struct iked *, struct sockaddr *);
 void vroute_removeroute(struct iked *, struct sockaddr *);
+#ifdef WITH_SYSTEMD
+int vroute_dbus_default_route(struct iked *, int, sd_bus_error *, int,
+    const char *, const char *, const char *);
+int vroute_dbus_dns(struct iked *, int, sd_bus_error *, int, const char *,
+    const char *, const char *);
+#endif
 
 struct vroute_addr {
 	int				va_ifidx;
@@ -62,20 +72,24 @@ struct vroute_route {
 };
 TAILQ_HEAD(vroute_routes, vroute_route);
 
-struct vroute_iface {
-	unsigned int			vi_ifidx;
-	TAILQ_ENTRY(vroute_iface)	vi_entry;
+struct vroute_dns {
+	struct	sockaddr_storage	vd_addr;
+	int				vd_ifidx;
+	TAILQ_ENTRY(vroute_dns)		vd_entry;
 };
-TAILQ_HEAD(vroute_ifaces, vroute_iface);
+TAILQ_HEAD(vroute_dnss, vroute_dns);
 
 static int	nl_addattr(struct nlmsghdr *, int, void *, size_t);
 static int	nl_dorule(struct iked *, int, uint32_t, int, int);
 
 struct iked_vroute_sc {
-	struct vroute_addrs	ivr_addrs;
-	struct vroute_ifaces	ivr_ifaces;
-	struct vroute_routes	ivr_routes;
-	int			ivr_rtsock;
+	struct vroute_addrs	 ivr_addrs;
+	struct vroute_dnss	 ivr_dnss;
+	struct vroute_routes	 ivr_routes;
+	int			 ivr_rtsock;
+#ifdef WITH_SYSTEMD
+	sd_bus			*ivr_bus;
+#endif
 };
 
 #define NL_BUFLEN	1024
@@ -95,8 +109,17 @@ vroute_init(struct iked *env)
 	    NETLINK_ROUTE)) == -1)
 		fatal("%s: failed to create netlink socket", __func__);
 
+#ifdef WITH_SYSTEMD
+	int r;
+	r = sd_bus_open_system(&ivr->ivr_bus);
+	if (r < 0) {
+		log_warn("%s: sd_bus_open_system", __func__);
+		ivr->ivr_bus = NULL;
+	}
+#endif
+
 	TAILQ_INIT(&ivr->ivr_addrs);
-	TAILQ_INIT(&ivr->ivr_ifaces);
+	TAILQ_INIT(&ivr->ivr_dnss);
 	TAILQ_INIT(&ivr->ivr_routes);
 
 	env->sc_vroute = ivr;
@@ -109,8 +132,8 @@ vroute_cleanup(struct iked *env)
 {
 	struct iked_vroute_sc	*ivr = env->sc_vroute;
 	struct vroute_addr	*addr;
-	struct vroute_iface	*iface;
 	struct vroute_route	*route;
+	struct vroute_dns	*dns;
 
 	while ((addr = TAILQ_FIRST(&ivr->ivr_addrs))) {
 		vroute_doaddr(env, addr->va_ifidx,
@@ -120,18 +143,18 @@ vroute_cleanup(struct iked *env)
 		free(addr);
 	}
 
-	while ((iface = TAILQ_FIRST(&ivr->ivr_ifaces))) {
-		vroute_dodns(env, NULL, 0, iface->vi_ifidx);
-		TAILQ_REMOVE(&ivr->ivr_ifaces, iface, vi_entry);
-		free(iface);
-	}
-
 	while ((route = TAILQ_FIRST(&ivr->ivr_routes))) {
 		vroute_doroute(env, RTM_DELROUTE,
 		    (struct sockaddr *)&route->vr_dest, NULL, NULL, 1);
 		TAILQ_REMOVE(&ivr->ivr_routes, route, vr_entry);
 		free(route);
 	}
+
+	while ((dns = TAILQ_FIRST(&ivr->ivr_dnss))) {
+		vroute_dodns(env, 0, dns->vd_ifidx);
+		TAILQ_REMOVE(&ivr->ivr_dnss, dns, vd_entry);
+		free(dns);
+	}
 }
 
 int
@@ -277,40 +300,13 @@ vroute_getdns(struct iked *env, struct imsg *imsg)
 	left -= sizeof(ifidx);
 
 	add = (imsg->hdr.type == IMSG_VDNS_ADD);
-	return (vroute_dodns(env, dns, add, ifidx));
-}
-
-void
-vroute_insertiface(struct iked *env, unsigned int ifidx)
-{
-	struct iked_vroute_sc	*ivr = env->sc_vroute;
-	struct vroute_iface	*iface;
-
-	/* Prevent duplicates */
-	TAILQ_FOREACH(iface, &ivr->ivr_ifaces, vi_entry) {
-		if (iface->vi_ifidx == ifidx)
-			return;
+	if (add) {
+		vroute_insertdns(env, ifidx, dns);
+	} else {
+		vroute_removedns(env, ifidx, dns);
 	}
 
-	iface = calloc(1, sizeof(*iface));
-	if (iface == NULL)
-		fatalx("%s: calloc.", __func__);
-
-	iface->vi_ifidx = ifidx;
-	TAILQ_INSERT_TAIL(&ivr->ivr_ifaces, iface, vi_entry);
-}
-
-void
-vroute_removeiface(struct iked *env, unsigned int ifidx)
-{
-	struct iked_vroute_sc	*ivr = env->sc_vroute;
-	struct vroute_iface	*iface;
-
-	TAILQ_FOREACH(iface, &ivr->ivr_ifaces, vi_entry) {
-		if (iface->vi_ifidx == ifidx)
-			continue;
-		TAILQ_REMOVE(&ivr->ivr_ifaces, iface, vi_entry);
-	}
+	return (vroute_dodns(env, add, ifidx));
 }
 
 void
@@ -341,6 +337,39 @@ vroute_removeroute(struct iked *env, struct sockaddr *dest)
 	}
 }
 
+void
+vroute_insertdns(struct iked *env, int ifidx, struct sockaddr *addr)
+{
+	struct iked_vroute_sc	*ivr = env->sc_vroute;
+	struct vroute_dns	*dns;
+
+	dns = calloc(1, sizeof(*dns));
+	if (dns == NULL)
+		fatalx("%s: calloc.", __func__);
+
+	memcpy(&dns->vd_addr, addr, SA_LEN(addr));
+	dns->vd_ifidx = ifidx;
+
+	TAILQ_INSERT_TAIL(&ivr->ivr_dnss, dns, vd_entry);
+}
+
+void
+vroute_removedns(struct iked *env, int ifidx, struct sockaddr *addr)
+{
+	struct iked_vroute_sc	*ivr = env->sc_vroute;
+	struct vroute_dns	*dns, *tdns;
+
+	TAILQ_FOREACH_SAFE(dns, &ivr->ivr_dnss, vd_entry, tdns) {
+		if (ifidx != dns->vd_ifidx)
+			continue;
+		if (sockaddr_cmp(addr, (struct sockaddr *) &dns->vd_addr, -1))
+			continue;
+
+		TAILQ_REMOVE(&ivr->ivr_dnss, dns, vd_entry);
+		free(dns);
+	}
+}
+
 void
 vroute_insertaddr(struct iked *env, int ifidx, struct sockaddr *addr,
     struct sockaddr *mask)
@@ -711,37 +740,43 @@ vroute_doaddr(struct iked *env, int ifidx, struct sockaddr *addr,
 }
 
 int
-vroute_dodns(struct iked *env, struct sockaddr *dns, int add,
-    unsigned int ifindex)
+vroute_dodns(struct iked *env, int add, unsigned int ifindex)
 {
-	if (add)
-		vroute_insertiface(env, ifindex);
-	else
-		vroute_removeiface(env, ifindex);
+#ifdef WITH_SYSTEMD
+	const char *destination = "org.freedesktop.resolve1";
+	const char *path = "/org/freedesktop/resolve1";
+	const char *interface = "org.freedesktop.resolve1.Manager";
+	sd_bus_error error = SD_BUS_ERROR_NULL;
+	int ret;
 
-#ifdef HAVE_SYSTEMD
-	char ifname[IF_NAMESIZE];
+	ret = vroute_dbus_dns(env, ifindex, &error, add,
+	    destination, path, interface);
+	if (ret < 0 && sd_bus_error_has_name(&error,
+	    "org.freedesktop.resolve1.LinkBusy")) {
 
-	if (if_indextoname(ifindex, ifname) == NULL)
-		return (-1);
+		/*
+                 * Interface is managed by networkd, so we need to send
+                 * our messages there instead.
+                 */
+		destination = "org.freedesktop.network1";
+		path = "/org/freedesktop/network1";
+		interface = "org.freedesktop.network1.Manager";
 
-#define resolvectl(...) do {				\
-		if (fork() == 0) {			\
-			execl("/usr/bin/resolvectl",	\
-			    __VA_ARGS__);		\
-			exit(0);			\
-		}					\
-	} while (0)
-
-	if (add && dns) {
-		resolvectl("/usr/bin/resolvectl", "dns", ifname,
-		    print_host(dns, NULL, 0), (char *) NULL);
-
-		resolvectl("/usr/bin/resolvectl", "domain", ifname,
-		    "~.", (char *) NULL);
-	} else {
-		resolvectl("/usr/bin/resolvectl", "revert", ifname,
-		    (char *) NULL);
+		sd_bus_error_free(&error);
+		ret = vroute_dbus_dns(env, ifindex, &error, add,
+		    destination, path, interface);
+	}
+	if (ret < 0) {
+		log_info("%s: vroute_dbus_dns: %s: %s", __func__,
+		    error.name, error.message);
+		sd_bus_error_free(&error);
+	}
+	ret = vroute_dbus_default_route(env, ifindex, &error, add,
+	    destination, path, interface);
+	if (ret < 0) {
+		log_info("%s: vroute_dbus_default_route: %s: %s", __func__,
+		    error.name, error.message);
+		sd_bus_error_free(&error);
 	}
 #endif
 	return (0);
@@ -798,3 +833,89 @@ nl_addattr(struct nlmsghdr *hdr, int type, void *data, size_t len)
 	hdr->nlmsg_len = NLMSG_ALIGN(hdr->nlmsg_len) + RTA_ALIGN(rta->rta_len);
 	return (0);
 }
+
+#ifdef WITH_SYSTEMD
+int
+vroute_dbus_default_route(struct iked *env, int ifidx, sd_bus_error *error, int add,
+    const char *destination, const char *path, const char *interface)
+{
+	struct iked_vroute_sc *ivr = env->sc_vroute;
+	sd_bus *bus = ivr->ivr_bus;
+	int ret;
+
+	ret = sd_bus_call_method(bus, destination, path, interface,
+	    "SetLinkDefaultRoute", error, NULL, "ib", ifidx, add);
+	return (ret);
+}
+
+int
+vroute_dbus_dns(struct iked *env, int ifidx, sd_bus_error *error, int add,
+    const char *destination, const char *path, const char *interface)
+{
+	struct sockaddr *dns;
+	struct sockaddr_in *in;
+	struct sockaddr_in6 *in6;
+	struct iked_vroute_sc *ivr = env->sc_vroute;
+	struct vroute_dns *vdns;
+	sd_bus_message *req = NULL;
+	sd_bus *bus = ivr->ivr_bus;
+	int ret;
+
+	ret = sd_bus_message_new_method_call(bus, &req,
+	    destination, path, interface, "SetLinkDNS");
+	if (ret < 0)
+		return (-1);
+
+	ret = sd_bus_message_append(req, "i", ifidx);
+	if (ret < 0)
+		return (-1);
+
+	ret = sd_bus_message_open_container(req, 'a', "(iay)");
+	if (ret < 0)
+		return (-1);
+
+	/* Empty list means remove all */
+	if (add) {
+		TAILQ_FOREACH(vdns, &ivr->ivr_dnss, vd_entry) {
+			if (vdns->vd_ifidx != ifidx)
+				continue;
+
+			ret = sd_bus_message_open_container(req, 'r', "iay");
+			if (ret < 0)
+				return (-1);
+
+			dns = (struct sockaddr *)&vdns->vd_addr;
+			ret = sd_bus_message_append(req, "i", dns->sa_family);
+			if (ret < 0)
+				return (-1);
+
+			switch (dns->sa_family) {
+			case AF_INET:
+				in = (struct sockaddr_in *)dns;
+				ret = sd_bus_message_append_array(req, 'y',
+				    &in->sin_addr, 4);
+				if (ret < 0)
+					return (-1);
+				break;
+			case AF_INET6:
+				in6 = (struct sockaddr_in6 *)dns;
+				ret = sd_bus_message_append_array(req, 'y',
+				    &in6->sin6_addr, 16);
+				if (ret < 0)
+					return (-1);
+				break;
+			}
+			ret = sd_bus_message_close_container(req);
+			if (ret < 0)
+				return (-1);
+		}
+	}
+
+	ret = sd_bus_message_close_container(req);
+	if (ret < 0)
+		return (-1);
+
+	ret = sd_bus_call(bus, req, 0, error, NULL);
+	return (ret);
+}
+#endif /* WITH_SYSTEMD */
