@@ -16,6 +16,8 @@
 #include "iked.h"
 #include "types.h"
 
+#define IKEV2_FLAG_INITIATOR            0x08    /* Sent by the initiator */
+
 int	 eap_parse(struct iked *, const struct iked_sa *,
 	    struct iked_message *, void *, int);
 int	 ikev2_msg_frompeer(struct iked_message *);
@@ -59,9 +61,26 @@ eap_parse(struct iked *env, const struct iked_sa *sa, struct iked_message *msg,
 	return (0);
 }
 
+/* Copied from ikev2_msg.c for better coverage */
 int
 ikev2_msg_frompeer(struct iked_message *msg)
 {
+	struct iked_sa		*sa = msg->msg_sa;
+	struct ike_header	*hdr;
+
+	msg = msg->msg_parent;
+
+	if (sa == NULL ||
+	    (hdr = ibuf_seek(msg->msg_data, 0, sizeof(*hdr))) == NULL)
+		return (0);
+
+	if (!sa->sa_hdr.sh_initiator &&
+	    (hdr->ike_flags & IKEV2_FLAG_INITIATOR))
+		return (1);
+	else if (sa->sa_hdr.sh_initiator &&
+	    (hdr->ike_flags & IKEV2_FLAG_INITIATOR) == 0)
+		return (1);
+
 	return (0);
 }
 
@@ -225,25 +244,30 @@ ikev2_msg_lookup(struct iked *env, struct iked_msgqueue *queue,
 	return NULL;
 }
 
+/* copied from ikev2_msg.c */
 void
 ikev2_msg_cleanup(struct iked *env, struct iked_message *msg)
 {
 	struct iked_certreq *cr;
 	struct iked_proposal *prop, *proptmp;
+	int			 i;
 
 	if (msg == msg->msg_parent) {
-		ibuf_release(msg->msg_nonce);
-		ibuf_release(msg->msg_ke);
-		ibuf_release(msg->msg_auth.id_buf);
-		ibuf_release(msg->msg_peerid.id_buf);
-		ibuf_release(msg->msg_localid.id_buf);
-		ibuf_release(msg->msg_cert.id_buf);
-		ibuf_release(msg->msg_cookie);
-		ibuf_release(msg->msg_cookie2);
-		ibuf_release(msg->msg_del_buf);
+		ibuf_free(msg->msg_nonce);
+		ibuf_free(msg->msg_ke);
+		ibuf_free(msg->msg_auth.id_buf);
+		ibuf_free(msg->msg_peerid.id_buf);
+		ibuf_free(msg->msg_localid.id_buf);
+		ibuf_free(msg->msg_cert.id_buf);
+		for (i = 0; i < IKED_SCERT_MAX; i++)
+			ibuf_free(msg->msg_scert[i].id_buf);
+		ibuf_free(msg->msg_cookie);
+		ibuf_free(msg->msg_cookie2);
+		ibuf_free(msg->msg_del_buf);
 		free(msg->msg_eap.eam_user);
 		free(msg->msg_cp_addr);
 		free(msg->msg_cp_addr6);
+		free(msg->msg_cp_dns);
 
 		TAILQ_FOREACH_SAFE(prop, &msg->msg_proposals, prop_entry,
 		    proptmp) {
@@ -253,15 +277,31 @@ ikev2_msg_cleanup(struct iked *env, struct iked_message *msg)
 			free(prop);
 		}
 
+		msg->msg_nonce = NULL;
+		msg->msg_ke = NULL;
+		msg->msg_auth.id_buf = NULL;
+		msg->msg_peerid.id_buf = NULL;
+		msg->msg_localid.id_buf = NULL;
+		msg->msg_cert.id_buf = NULL;
+		for (i = 0; i < IKED_SCERT_MAX; i++)
+			msg->msg_scert[i].id_buf = NULL;
+		msg->msg_cookie = NULL;
+		msg->msg_cookie2 = NULL;
+		msg->msg_del_buf = NULL;
+		msg->msg_eap.eam_user = NULL;
+		msg->msg_cp_addr = NULL;
+		msg->msg_cp_addr6 = NULL;
+		msg->msg_cp_dns = NULL;
+
 		while ((cr = SIMPLEQ_FIRST(&msg->msg_certreqs))) {
-			ibuf_release(cr->cr_data);
+			ibuf_free(cr->cr_data);
 			SIMPLEQ_REMOVE_HEAD(&msg->msg_certreqs, cr_entry);
 			free(cr);
 		}
 	}
 
 	if (msg->msg_data != NULL) {
-		ibuf_release(msg->msg_data);
+		ibuf_free(msg->msg_data);
 		msg->msg_data = NULL;
 	}
 }
